//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
#property copyright "mladen"
#property link      "www.forex-station.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1  clrLimeGreen
#property indicator_color2  clrRed
#property indicator_color3  clrRed
#property indicator_color4  clrDeepSkyBlue  
#property indicator_color5  clrDimGray
#property indicator_color6  clrPaleVioletRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_style4  STYLE_DOT
#property indicator_style5  STYLE_DOT
#property indicator_style6  STYLE_DOT

//
//
//
//
//

enum enRsiTypes
{
   rsi_rsi,  // Regular RSI
   rsi_wil,  // Wilders' RSI
   rsi_rap,  // Rapid RSI
   rsi_har,  // Harris RSI
   rsi_rsx,  // RSX
   rsi_cut   // Cuttlers RSI
};
enum colorOn
{
   clrOnSlope, // Color change on slope change
   clrOnZero,  // Color change on zero cross
   clrOnlevel  // Color change on levels cross
};


extern ENUM_TIMEFRAMES TimeFrame        = PERIOD_CURRENT;   // Time frame to use
extern int             RsiPeriod        = 14;
extern enRsiTypes      RsiType          = rsi_rsx;          // Rsi type
extern int             AmaPeriod        = 10;
extern int             AmaPrice         = PRICE_CLOSE;
extern double          FastEnd          = 2;
extern double          SlowEnd          = 30;
extern double          SmoothPower      = 2;
extern bool            JurikFDAdaptive  = true;
extern int             MinMaxPeriod     = 35;
extern double          LevelUp          = 90;
extern double          LevelDown        = 10;
extern colorOn         ColorChangeOn    = clrOnSlope;             // Color change on : 
extern bool            Interpolate      = true;

//
//
//
//
//

double rsiLUp[];
double rsiLMi[];
double rsiLDn[];
double rsi[];
double rsiUpa[];
double rsiUpb[];
double rsiDna[];
double rsiDnb[];
double ama[];
double trend[];
double trenc[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
// 
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,rsi,   INDICATOR_DATA);
   SetIndexBuffer(1,rsiDna,INDICATOR_DATA);
   SetIndexBuffer(2,rsiDnb,INDICATOR_DATA);
   SetIndexBuffer(3,rsiLUp,INDICATOR_DATA);
   SetIndexBuffer(4,rsiLMi,INDICATOR_DATA);
   SetIndexBuffer(5,rsiLDn,INDICATOR_DATA);
   SetIndexBuffer(6,ama   ,INDICATOR_CALCULATIONS);
   SetIndexBuffer(7,trend ,INDICATOR_CALCULATIONS);
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
   
      IndicatorShortName(timeFrameToString(TimeFrame)+"  "+getRsiName((int)RsiType)+" of kaufman ("+(string)RsiPeriod+")");
   return(0);
}
int deinit()
{
   
   return(0); 
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double work[][2];
#define _lastLo 0
#define _lastHi 1

int start()
{
   int i,limit,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { rsi[0] = limit+1; return(0); }
         if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);

   //
   //
   //
   //
   //

   if (TimeFrame == Period())
   { 
      if (trend[limit] ==-1) CleanPoint(limit,rsiDna,rsiDnb);
      for(i=limit; i>=0; i--)  ama[i] = iKama(AmaPeriod,FastEnd,SlowEnd,AmaPrice,SmoothPower,JurikFDAdaptive,i);
      for(i=limit; i>=0; i--)
      {
            rsi[i]    = iRsi(ama[i],RsiPeriod,RsiType,i);
           
            //
            //
            //
            //
            //
                   
            double rsiMin = rsi[ArrayMinimum(rsi,MinMaxPeriod,i)];
            double rsiMax = rsi[ArrayMaximum(rsi,MinMaxPeriod,i)];
            double range  = rsiMax-rsiMin;
               rsiLDn[i]  = rsiMin+range*LevelDown/100.0;
               rsiLMi[i]  = rsiMin+range*0.5;
               rsiLUp[i]  = rsiMin+range*LevelUp/100.0;
               
               rsiDna[i] = EMPTY_VALUE;
               rsiDnb[i] = EMPTY_VALUE;
               trend[i]  = trend[i+1];
               switch (ColorChangeOn)
               {
                  case clrOnSlope: 
                     if (rsi[i]>rsi[i+1])    trend[i] =  1;
                     if (rsi[i]<rsi[i+1])    trend[i] = -1;
                  break;
                  case clrOnZero: 
                     if (rsi[i]>rsiLMi[i])   trend[i] =  1;
                     if (rsi[i]<rsiLMi[i])   trend[i] = -1;
                  break;
                  default : 
                     if (rsi[i]>rsiLUp[i])   trend[i] =  1;
                     if (rsi[i]<rsiLDn[i])   trend[i] = -1;
                  break;
               }    
               if (trend[i] == -1) PlotPoint(i,rsiDna,rsiDnb,rsi);
      }      
   return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = (int)MathMax(limit,MathMin(Bars,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   if (trend[limit] ==-1) CleanPoint(limit,rsiDna,rsiDnb);
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         rsi   [i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiType,AmaPeriod,AmaPrice,FastEnd,SlowEnd,SmoothPower,JurikFDAdaptive,MinMaxPeriod,LevelUp,LevelDown,ColorChangeOn,0,y); 
         rsiLUp[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiType,AmaPeriod,AmaPrice,FastEnd,SlowEnd,SmoothPower,JurikFDAdaptive,MinMaxPeriod,LevelUp,LevelDown,ColorChangeOn,3,y);
         rsiLMi[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiType,AmaPeriod,AmaPrice,FastEnd,SlowEnd,SmoothPower,JurikFDAdaptive,MinMaxPeriod,LevelUp,LevelDown,ColorChangeOn,4,y);
         rsiLDn[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiType,AmaPeriod,AmaPrice,FastEnd,SlowEnd,SmoothPower,JurikFDAdaptive,MinMaxPeriod,LevelUp,LevelDown,ColorChangeOn,5,y);
         trend[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiType,AmaPeriod,AmaPrice,FastEnd,SlowEnd,SmoothPower,JurikFDAdaptive,MinMaxPeriod,LevelUp,LevelDown,ColorChangeOn,7,y); 
         rsiDna[i] = EMPTY_VALUE;
         rsiDnb[i] = EMPTY_VALUE;
         
         //
         //
         //
         //
         //
      
         if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;
                  
         //
         //
         //
         //
         //
                  
         int n,j; datetime time = iTime(NULL,TimeFrame,y);
            for(n = 1; (i+n)<Bars && Time[i+n] >= time; n++) continue;	
            for(j = 1; j<n && (i+n)<Bars && (i+j)<Bars; j++)
            {
               rsi[i+j]    = rsi[i]    + (rsi[i+n]    - rsi[i]   )*j/n;
               rsiLUp[i+j] = rsiLUp[i] + (rsiLUp[i+n] - rsiLUp[i])*j/n;
               rsiLMi[i+j] = rsiLMi[i] + (rsiLMi[i+n] - rsiLMi[i])*j/n;
               rsiLDn[i+j] = rsiLDn[i] + (rsiLDn[i+n] - rsiLDn[i])*j/n;
            }
            
   }
   for (i=limit;i>=0;i--) if (trend[i] ==-1)PlotPoint(i,rsiDna,rsiDnb,rsi);           
return(0);       
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//
//

string rsiMethodNames[] = {"RSI","Wilders' RSI","Rapid RSI","Harris RSI","RSX","Cuttler RSI"};
string getRsiName(int method)
{
   int max = ArraySize(rsiMethodNames)-1;
      method=MathMax(MathMin(method,max),0); return(rsiMethodNames[method]);
}

//
//
//
//
//

double workRsi[][13];
#define _price  0
#define _change 1
#define _changa 2
#define _rsival 1
#define _rsval  1

double iRsi(double price, double period, int rsiMode, int i, int instanceNo=0)
{
   if (ArrayRange(workRsi,0)!=Bars) ArrayResize(workRsi,Bars);
      int z = instanceNo*13; 
      int r = Bars-i-1;
   
   //
   //
   //
   //
   //
   
   workRsi[r][z+_price] = price;
   switch (rsiMode)
   {
      case 0:
         double alpha = 1.0/period; 
         if (r<period)
            {
               int k; double sum = 0; for (k=0; k<period && (r-k-1)>=0; k++) sum += MathAbs(workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price]);
                  workRsi[r][z+_change] = (workRsi[r][z+_price]-workRsi[0][z+_price])/MathMax(k,1);
                  workRsi[r][z+_changa] =                                         sum/MathMax(k,1);
            }
         else
            {
               double change = workRsi[r][z+_price]-workRsi[r-1][z+_price];
                               workRsi[r][z+_change] = workRsi[r-1][z+_change] + alpha*(        change  - workRsi[r-1][z+_change]);
                               workRsi[r][z+_changa] = workRsi[r-1][z+_changa] + alpha*(MathAbs(change) - workRsi[r-1][z+_changa]);
            }
         if (workRsi[r][z+_changa] != 0)
               return(50.0*(workRsi[r][z+_change]/workRsi[r][z+_changa]+1));
         else  return(50.0);
         
      //
      //
      //
      //
      //
      
      case 1 :
         double up = 0;
         double dn = 0;
         for(k=0; k<(int)period && (r-k-1)>=0; k++)
         {
            double diff = workRsi[r-k][z+_price]- workRsi[r-k-1][z+_price];
            if(diff>0)
                  up += diff;
            else  dn -= diff;
         }
         if(up + dn == 0)
               workRsi[r][z+_rsival] = workRsi[r-1][z+_rsival]+(1/period)*(50            -workRsi[r-1][z+_rsival]);
         else  workRsi[r][z+_rsival] = workRsi[r-1][z+_rsival]+(1/period)*(100*up/(up+dn)-workRsi[r-1][z+_rsival]);
         return(workRsi[r][z+_rsival]);      

      //
      //
      //
      //
      //

      case 2 :
         up = 0;
         dn = 0;
         for(k=0; k<(int)period && (r-k-1)>=0; k++)
         {
            diff = workRsi[r-k][z+_price]- workRsi[r-k-1][z+_price];
            if(diff>0)
                  up += diff;
            else  dn -= diff;
         }
         if(up + dn == 0)
               return(50);
         else  return(100 * up / (up + dn));      

      //
      //
      //
      //
      //

      
      case 3 :
         double avgUp=0,avgDn=0; up=0;dn=0;
         for(k=0; k<(int)period && (r-k-1)>=0; k++)
         {
            diff = workRsi[r-k][instanceNo+_price]- workRsi[r-k-1][instanceNo+_price];
            if(diff>0)
                  { avgUp += diff; up++; }
            else  { avgDn -= diff; dn++; }
         }
         if (up!=0) avgUp /= up;
         if (dn!=0) avgDn /= dn;
         double rs = 1;
            if (avgDn!=0) rs = avgUp/avgDn;
            return(100-100/(1.0+rs));

      //
      //
      //
      //
      //
      
      case 4 :     
         double Kg = (3.0)/(2.0+period), Hg = 1.0-Kg;
         if (r<period) { for (k=1; k<13; k++) workRsi[r][k+z] = 0; return(50); }  

         //
         //
         //
         //
         //
      
         double mom = workRsi[r][_price+z]-workRsi[r-1][_price+z];
         double moa = MathAbs(mom);
         for (k=0; k<3; k++)
         {
            int kk = k*2;
            workRsi[r][z+kk+1] = Kg*mom                + Hg*workRsi[r-1][z+kk+1];
            workRsi[r][z+kk+2] = Kg*workRsi[r][z+kk+1] + Hg*workRsi[r-1][z+kk+2]; mom = 1.5*workRsi[r][z+kk+1] - 0.5 * workRsi[r][z+kk+2];
            workRsi[r][z+kk+7] = Kg*moa                + Hg*workRsi[r-1][z+kk+7];
            workRsi[r][z+kk+8] = Kg*workRsi[r][z+kk+7] + Hg*workRsi[r-1][z+kk+8]; moa = 1.5*workRsi[r][z+kk+7] - 0.5 * workRsi[r][z+kk+8];
         }
         if (moa != 0)
              return(MathMax(MathMin((mom/moa+1.0)*50.0,100.00),0.00)); 
         else return(50);
            
      //
      //
      //
      //
      //
      
      case 5 :
         double sump = 0;
         double sumn = 0;
         for (k=0; k<(int)period; k++)
         {
            diff = workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price];
               if (diff > 0) sump += diff;
               if (diff < 0) sumn -= diff;
         }
         if (sumn > 0)
               return(100.0-100.0/(1.0+sump/sumn));
         else  return(50);
   } 
return(0);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//

double workAma[][2];
#define _diff 0
#define _kama 1

//
//
//
//
//

double iKama(int period, double fast, double slow, int priceType, double power, bool JurikFD, int i)
{
   if (ArrayRange(workAma,0)!=Bars) ArrayResize(workAma,Bars);

   double price   = iMA(NULL,0,1,0,MODE_SMA,priceType,i); int r = Bars-i-1; if (r<period) return(price);
   double fastend = (2.0 /(fast + 1));
   double slowend = (2.0 /(slow + 1));
   
   //
   //
   //
   //
   //

      if (JurikFD) double efratio = MathMin(2.0-iJurikFractalDimension(period,2,i),1.0);
      else
      {
         double signal = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,priceType,i+period));
         double noise  = 0;
            workAma[r][_diff] = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,priceType,i+1));
            for (int k=0; k<period; k++) 
                  noise += workAma[r-k][_diff];

            //
            //
            //
            //
            //

            if (noise != 0)
                  efratio = signal/noise;
            else  efratio = 1;
      }
      double smooth = MathPow(efratio*(fastend-slowend)+slowend,power);
             workAma[r][_kama] = workAma[r-1][_kama] + smooth*(price-workAma[r-1][_kama]);

   //
   //
   //
   //
   //
      
   return(workAma[r][_kama]);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double workFd[][2];
#define _smlRange 0
#define _smlSumm  1

//
//
//
//
//

double iJurikFractalDimension(int size, int count,int i)
{
   if (ArrayRange(workFd,0)!=Bars) ArrayResize(workFd,Bars);

   int window1  = size*(count-1);
   int window2  = size* count;
   int r        = Bars-i-1;

   //
   //
   //
   //
   //

      workFd[r][_smlRange] = iFdRange(size,i);
      workFd[r][_smlSumm]  = workFd[r-1][_smlSumm]+workFd[r][_smlRange];
      
      if ((Bars-i)>window1)
      {
         workFd[r][_smlSumm] -= workFd[r-window1][_smlRange];
         return(2.0-MathLog(iFdRange(window2,i)/(workFd[r][_smlSumm]/window1))/MathLog(count));
      }
      else return(0.5);
}
double iFdRange(int size,int i) { return(MathMax(Close[size+i],High[ArrayMaximum(High,size,i)])-MathMin(Close[size+i],Low[ArrayMinimum(Low,size,i)])); } 

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}